
// Include necessary libraries for SPI communication and the nRF24L01 module
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

// Create an RF24 object for managing the nRF24L01 module, specifying the CE and CSN pins
RF24 radio(7, 8);
// Define the address for the radio communication. Both transmitter and receiver need to use the same address
const byte address[6] = "00001";

// Define analog input pins for the two joysticks (X and Y axes)
const int joystick1X = A1;
const int joystick1Y = A0;
const int joystick2X = A2;
const int joystick2Y = A3;

// Define a threshold for the dead zone to minimize noise around the center position
const int deadZone = 15;

// Smoothing factor for the exponential moving average (EMA)
const float smoothingFactor = 0.2;
// Variables to store the smoothed values
float smoothJ1X = 0;
float smoothJ1Y = 0;
float smoothJ2X = 0;
float smoothJ2Y = 0;

void setup() {
  // Initialize the nRF24L01 module
  radio.begin();
  // Open a writing pipe on the specified address
  radio.openWritingPipe(address);
  // Set the power amplifier level to minimum to save power and reduce interference
  radio.setPALevel(RF24_PA_MIN);
  // Stop listening on the modules to switch them to transmitter mode
  radio.stopListening();

  // Initialize joystick pins as inputs
  pinMode(joystick1X, INPUT);
  pinMode(joystick1Y, INPUT);
  pinMode(joystick2X, INPUT);
  pinMode(joystick2Y, INPUT);
}

void loop() {
  // Read the analog values from the joystick's X and Y axes and center them around 0
  int j1XValue = analogRead(joystick1X) - 512; // Center joystick1 X axis
  int j1YValue = analogRead(joystick1Y) - 512; // Center joystick1 Y axis
  int j2XValue = analogRead(joystick2X) - 512; // Center joystick2 X axis
  int j2YValue = analogRead(joystick2Y) - 512; // Center joystick2 Y axis

  // Apply a dead zone to reduce noise and unintended small movements
  j1XValue = (abs(j1XValue) < deadZone) ? 0 : j1XValue;
  j1YValue = (abs(j1YValue) < deadZone) ? 0 : j1YValue;
  j2XValue = (abs(j2XValue) < deadZone) ? 0 : j2XValue;
  j2YValue = (abs(j2YValue) < deadZone) ? 0 : j2YValue;

  // Apply exponential moving average smoothing
  smoothJ1X = smoothJ1X * (1 - smoothingFactor) + j1XValue * smoothingFactor;
  smoothJ1Y = smoothJ1Y * (1 - smoothingFactor) + j1YValue * smoothingFactor;
  smoothJ2X = smoothJ2X * (1 - smoothingFactor) + j2XValue * smoothingFactor;
  smoothJ2Y = smoothJ2Y * (1 - smoothingFactor) + j2YValue * smoothingFactor;

  // Prepare a structure to hold and transmit both joystick values
  struct {
    int j1X;
    int j1Y;
    int j2X;
    int j2Y;
  } joystickValues = {smoothJ1X, smoothJ1Y, smoothJ2X, smoothJ2Y};

  // Transmit the smoothed joystick values
  radio.write(&joystickValues, sizeof(joystickValues));

  // Delay to control the rate of data transmission
  delay(50); // Adjust this delay based on your specific needs
}
